//+------------------------------------------------------------------+
//|                                               Key of the day.mq4 |
//|                                     contactchristinali@gmail.com |
//|                               http//www.wix.com/wiseea/wise-ea#! | 
//+------------------------------------------------------------------+
#property copyright     "Programmed by Christina Li, Wise-EA MetaTrader Programming"
#property link          "www.wix.com/wiseea/wise-ea#! "
#property version       "1.00"
#property description   "Copyright Daniel Luchinger"
#property description   "Updated on 02/17/2014"
#property description   "Initial version, multiple time frame EA based on MA and KCP indicator"
//+------------------------------------------------------------------+
//| Setup & Include                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Input parameters                                                 |
//+------------------------------------------------------------------+
input string INDI               = " --- Indicator Settings --- "; 
input int    KC_MaPeriod        = 20;
input int    KC_MaMethod        = MODE_SMA;
input int    KC_PriceToUse      = PRICE_TYPICAL;
input int    KC_AtrPeriod       = 50;
input double KC_AtrMultiplier   = 2.5;
extern int   KC_MaAtrShift      = 1;
input int    MaPeriod           = 14;
input int    MaMethod           = MODE_EMA;
input int    MaPrice            = PRICE_CLOSE;
input string TRAD               = " --- Trade Opening Settings --- "; 
input int    HiTimeFrame        = 240;
input bool   UseCurrentBarHiTF  = false;
input int    MaxTradesAllowed   = 3;
input double StopLoss           = 50;
input double TakeProfit         = 50;
input double OpenLot            = 0.1;
input bool   UseAutoLots        = true;
input double Risk_Ratio         = 5;
input string MANAGE             = " --- Management & Exit Settings --- "; 
input bool   UseBE              = false;
input double BEPoint            = 25;
input double BEProfitPip        = 2;
input bool   UseTrailingStop    = false;
input double TrailActivation    = 30;
input double TrailingStop       = 20;
input double TrailingStep       = 1;
input string MISC               = " --- Miscellaneous Settings --- ";
input int    MagicNumber        = 763126; 
input double Slippage           = 2; 
input string ID                 = "LUCHI";
input bool   PlotArrow          = true;
input bool   SendAlerts         = true;
//+------------------------------------------------------------------+
//| Global variabels                                                 |
//+------------------------------------------------------------------+
int    dig, count, hibar;
double pnt;
string EAComment, sig, trend;
static datetime BarTime;
//+------------------------------------------------------------------+
//| EA initialization function                                       |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//---- initialize variables
   dig=SymbolInfoInteger(Symbol(),SYMBOL_DIGITS);
   pnt=SymbolInfoDouble(Symbol(),SYMBOL_POINT);
   if (dig==3 || dig==5) pnt*=10;   
   EAComment=ID+" "+Symbol()+string(Period());
   BarTime=Time[0];
   sig="X";
   hibar=1;
   if (UseCurrentBarHiTF) hibar=0;
   trend="Buy";
   if (ma(HiTimeFrame,hibar)<=kc(HiTimeFrame,hibar)) trend="Sell";
//----
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| EA deinitialization function                                     |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- 
   Comment("");
//--- 
  }  
//+------------------------------------------------------------------+
//| OnTick function                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//----
   manage_trades();
   if (isNewBar()) check_sig();
   manage_trades();
   if (sig=="Buy") opentrade(0);
   else if (sig=="Sell") opentrade(1);
   Comment("Trend direction on high time frame: ",trend);
//----
  }  
//******************************************************************************************************
//******************************************************************************************************
//+------------------------------------------------------------------+
//| FUNCTIONS: Check keltner indicator                               |
//+------------------------------------------------------------------+
double kc(int tf, int i) 
  {
//----
   double val=iCustom(NULL,tf,"Keltner channel_alerts 2.01",
              KC_MaPeriod,KC_MaMethod,KC_PriceToUse,KC_AtrPeriod,KC_AtrMultiplier,KC_MaAtrShift,"","","","","",false,false,false,false,false,
              4,i);
//----
   return (val);
  }     
//+------------------------------------------------------------------+
//| FUNCTIONS: Check ma indicator                                    |
//+------------------------------------------------------------------+
double ma(int tf, int i) 
  {
//----
   double val=iMA(NULL,tf,MaPeriod,0,MaMethod,MaPrice,i);
//----
   return (val);
  }               
//+------------------------------------------------------------------+
//| FUNCTIONS: Check signal                                          |
//+------------------------------------------------------------------+
void check_sig() 
  {
//----
   trend="Buy";
   if (ma(HiTimeFrame,hibar)<=kc(HiTimeFrame,hibar)) trend="Sell";
//----
   sig="X";
   if (ma(0,1)>kc(0,1) && ma(0,2)<=kc(0,2) && trend=="Buy")
     {
      sig="Buy";
      if (PlotArrow) plot_arrow(1,0,233);
      if (SendAlerts) Alert(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Buy signal identified."); 
      if (MaxTradesAllowed>0 && count>=MaxTradesAllowed) sig="X";  
     }
   else if (ma(0,1)<=kc(0,1) && ma(0,2)>kc(0,2) && trend=="Sell") 
     {
      sig="Sell";
      if (PlotArrow) plot_arrow(1,1,234); 
      if (SendAlerts) Alert(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Sell signal identified."); 
      if (MaxTradesAllowed>0 && count>=MaxTradesAllowed) sig="X";  
     }
  }
//******************************************************************************************************
//******************************************************************************************************  
//+------------------------------------------------------------------+
//| FUNCTIONS: Manage trades                                         |
//+------------------------------------------------------------------+
void manage_trades() 
  {
//----
   count=0;
   for (int i=OrdersTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber) continue;
      //----
      count++;
      if (UseBE) check_breakeven();
      if (UseTrailingStop) check_trailingstop();
     }
  }  
//+------------------------------------------------------------------+
//| FUNCTIONS: Trailing stop                                         |
//+------------------------------------------------------------------+  
void check_trailingstop() 
  {
//----
   if (OrderType()==0 && Bid>=OrderOpenPrice()+TrailActivation*pnt) 
     {
      if (OrderStopLoss()==0 || OrderStopLoss()<Bid-TrailingStop*pnt-TrailingStep*pnt)    
         OrderModify(OrderTicket(),OrderOpenPrice(),Bid-TrailingStop*pnt,OrderTakeProfit(),0,Lime);
     } 
   else if (OrderType()==1 && Ask<=OrderOpenPrice()-TrailActivation*pnt) 
     {
      if (OrderStopLoss()==0 || OrderStopLoss()>Ask+TrailingStop*pnt+TrailingStep*pnt) 
         OrderModify(OrderTicket(),OrderOpenPrice(),Ask+TrailingStop*pnt,OrderTakeProfit(),0,Red);
     }
  }   
//+------------------------------------------------------------------+
//| FUNCTIONS: Check break even                                      |
//+------------------------------------------------------------------+  
void check_breakeven() 
  {
//----
   if (OrderType()==0) 
     {
      double sl=NormalizeDouble(OrderOpenPrice()+BEProfitPip*pnt,dig);
      if ((OrderStopLoss()==0 || NormalizeDouble(OrderStopLoss(),dig)<sl) && (Bid>=OrderOpenPrice()+BEPoint*pnt)) 
        { 
         bool result=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,CLR_NONE);
         if (result)
           {
            string com=TimeToStr(TimeCurrent(), TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Order ("+OrderTicket()+") set to BreakEven : "+" @"+DoubleToStr(sl,dig);
            if (SendAlerts) Alert(com);
            else Print(com);
           }
         else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+"Error set to BreakEven : "+GetLastError());
        }
     }          
   else if (OrderType()==1) 
     {
      sl=NormalizeDouble(OrderOpenPrice()-BEProfitPip*pnt,dig);
      if ((OrderStopLoss()==0 || NormalizeDouble(OrderStopLoss(),dig)>sl) && (Ask<=OrderOpenPrice()-BEPoint*pnt)) 
        { 
         result=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,CLR_NONE);
         if (result)
           {
            com=TimeToStr(TimeCurrent(), TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Order ("+OrderTicket()+") set to BreakEven : "+" @"+DoubleToStr(sl,dig);
            if (SendAlerts) Alert(com);
            else Print(com); 
           }
         else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+"Error set to BreakEven : "+GetLastError());
        }
     }   
  }
//******************************************************************************************************
//******************************************************************************************************  
//+------------------------------------------------------------------+
//| FUNCTIONS: Order Excution                                        |
//+------------------------------------------------------------------+
void opentrade(int ordertype) 
  {
//----
   string typestring; 
   double price,stoploss,takeprofit;
   color  clr=Lime;
   if (ordertype==1 || ordertype==3 || ordertype==5) clr=Red;
//----
   if (ordertype==0) 
     {
      price=Ask;
      typestring=" BUY ";
     }
   else if (ordertype==1) 
     {
      price=Bid;
      typestring=" SELL ";
     }
//----  
   int ticket=OrderSend(Symbol(),ordertype,LotsOptimized(),price,Slippage,0,0,EAComment,MagicNumber,0,clr);
   if(ticket>0 && OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
     {
      string com=TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+typestring+" order ("+OrderTicket()+") opened: "+" @"+DoubleToStr(OrderOpenPrice(),dig);
      if (SendAlerts) Alert(com);
      else Print(com); 
      sig="OrderEntered";
      //----
      if (ordertype==0) 
        {
         if (StopLoss!=0) stoploss=OrderOpenPrice()-StopLoss*pnt;
         else stoploss=0;
         if (TakeProfit!=0) takeprofit=OrderOpenPrice()+TakeProfit*pnt;
         else takeprofit=0;
        }
      else if (ordertype==1) 
        {
         if (StopLoss!=0) stoploss=OrderOpenPrice()+StopLoss*pnt;
         else stoploss=0;
         if (TakeProfit!=0) takeprofit=OrderOpenPrice()-TakeProfit*pnt;
         else takeprofit=0;
        } 
      if (takeprofit!=0 || stoploss!=0) 
        {     
         for (int j=5; j>0; j--) 
           {
            OrderModify(ticket,OrderOpenPrice(),stoploss,takeprofit,0,CLR_NONE);
            int errnum=GetLastError();
            if (errnum==1) errnum=0;
            if (errnum==0) break;
            Sleep(1000);  
            RefreshRates();
           }
        } 
     }
   else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Error opening order : ",GetLastError()); 
  } 
//+------------------------------------------------------------------+
//| FUNCTIONS: Money Management                                      |
//+------------------------------------------------------------------+
double LotsOptimized() 
  {
//----
   double step=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
   double minlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   double maxlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
//----
   double val=0;
   if (UseAutoLots) 
     {   
      if (step==0.01) int digit=2;
      else if (step==0.1) digit=1;
      else digit=0;
      //----
      double maxloss=AccountInfoDouble(ACCOUNT_BALANCE)*Risk_Ratio/100;   
      double pointcost=SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_VALUE);
      double stoppoint=MathAbs(StopLoss*pnt/Point);
      if (stoppoint*pointcost>0) val=maxloss/(stoppoint*pointcost); 
      double nval=NormalizeDouble(val,digit);
      if (nval<=val) val=nval;
      else val=nval-step;
     }
   else val=OpenLot;
//---- 
   if (val<minlot) val=minlot;
   else if (val>maxlot) val=maxlot;
   return(val);  
  }
//******************************************************************************************************
//******************************************************************************************************
//+------------------------------------------------------------------+
//| FUNCTIONS: Plot signal arrow                                     |
//+------------------------------------------------------------------+  
void plot_arrow(int i, int type, int code) 
  {
//----
   string objname=EAComment+"_"+TimeToStr(Time[i])+"_SIG";
   if (type==0) 
     {
      double location=Low[i]-2*pnt;
      double clr=Lime;
     }
   else if (type==1) 
     {
      location=High[i]+5*pnt;
      clr=Red;
     }
//----
   if (ObjectFind(0,objname)==-1) 
     {      
      ObjectCreate(0,objname,OBJ_ARROW,0,Time[i],location); 
      ObjectSet(objname,OBJPROP_ARROWCODE,code);
      ObjectSet(objname,OBJPROP_COLOR,clr);
      ObjectSet(objname,OBJPROP_WIDTH,1);
      ObjectSet(objname,OBJPROP_TIMEFRAMES,0);
     }
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: New bar                                               |
//+------------------------------------------------------------------+
bool isNewBar() 
   {
//---- 
   bool res=false; 
   if (BarTime!=Time[0]) {
      BarTime=Time[0];
      sig="X";  
      res=true;
   } 
//----    
   return(res);   
}  